/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.controller;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import ru.ulstu.extractor.repository.CommitRepository;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;
import java.util.stream.Collectors;

import static ru.ulstu.extractor.controller.Route.STATISTIC;

@Controller
@ApiIgnore
public class StatisticController {
    private final CommitRepository commitRepository;

    public StatisticController(CommitRepository commitRepository) {
        this.commitRepository = commitRepository;
    }

    @GetMapping(STATISTIC)
    public String indexBranch(Model model) {
        List<Object[]> urlCommits = getUrlCommits();
        List<Object[]> timeCommits = getTimeCommits();
        model.addAttribute("commitAuthorData", authorCommits());
        model.addAttribute("commitUrlData", urlCommits);
        model.addAttribute("commitTimeData", timeCommits);
        List<String> dates = timeCommits.stream()
                .map(tc -> tc[0].toString())
                .collect(Collectors.toList());
        model.addAttribute("dates", dates);

        List<String> urls = urlCommits.stream()
                .map(tc -> tc[0].toString().substring(tc[0].toString().lastIndexOf("/") + 1))
                .collect(Collectors.toList());
        model.addAttribute("urls", urls);
        model.addAttribute("commitEntityData", getEntityCommits());
        model.addAttribute("commitTimeEntityData", allTimeEntityCommits());
        return STATISTIC;
    }

    public List<Object[]> authorCommits() {
        return commitRepository.getCommitAuthorStatistic().stream()
                .map(stat -> new Object[]{stat.getAuthor(), stat.getCountCommit()})
                .collect(Collectors.toList());
    }

    public List<Object[]> getUrlCommits() {
        return commitRepository.getCommitUrlStatistic().stream()
                .map(stat -> new Object[]{stat.getUrl(), stat.getCountCommit()})
                .collect(Collectors.toList());
    }

    public List<Object[]> getTimeCommits() {
        return commitRepository.getCommitTimeStatistic().stream()
                .map(stat -> new Object[]{stat.getDate(), stat.getCountCommit()})
                .collect(Collectors.toList());
    }

    public List<Object[]> getEntityCommits() {
        List<Object[]> entityCommits = commitRepository.getCommitEntityStatistic().stream()
                .map(stat -> new Object[]{stat.getEntity(), stat.getCountCommit()})
                .collect(Collectors.toList());
        for (Object[] entityCommit : entityCommits) {
            entityCommit[0] = Boolean.TRUE.equals(entityCommit[0])
                    ? "Есть сущности"
                    : (entityCommit[0] == null
                    ? "Нет данных"
                    : "Нет сущностей");
        }
        return entityCommits;
    }

    public List<Object[]> getTimeEntityCommits() {
        return commitRepository.getCommitTimeEntityStatistic().stream()
                .map(stat -> new Object[]{stat.getDate(), stat.getCountCommit()})
                .collect(Collectors.toList());
    }

    public List<Object[]> allTimeEntityCommits() {
        List<Object[]> timeEntityCommits = getTimeEntityCommits();
        List<Object[]> allTimeEntityCommits = commitRepository.getCommitTimeStatistic().stream()
                .map(stat -> new Object[]{stat.getDate(), stat.getCountCommit()})
                .collect(Collectors.toList());
        int j = 0;
        for (int i = 0; i < allTimeEntityCommits.size(); i++) {
            if ((j < timeEntityCommits.size()) && (allTimeEntityCommits.get(i)[0].equals(timeEntityCommits.get(j)[0]))) {
                allTimeEntityCommits.get(i)[1] = timeEntityCommits.get(j)[1];
                j++;
            } else {
                if (i > 0) {
                    allTimeEntityCommits.get(i)[1] = allTimeEntityCommits.get(i - 1)[1];
                }
            }
        }
        return allTimeEntityCommits;
    }
}
